/*
** image.c -- image file handling and manipulation routines for ShowTime
**
** Copyright (C) 1994 by Michael J. Gourlay
**
** NO WARRANTEES, EXPRESS OR IMPLIED.
*/

#include <fcntl.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "showtime.h"
#include "video.h"
#include "cmap.h"
#include "pbm.h"
#include "pic.h"
#include "tga.h"
#include "file.h"

#include "image.h"

/* ---------------------------------------------------------------------- */

/* image header initializer record */
image_hdr_t gm_hdr_init = {
  NO_IMAGE_MAGIC,  /* image_type */
  0,               /* num_rows */
  0,               /* num_cols */
  NULL,            /* load_img */
  NULL             /* hdr */
};

/* ====================================================================== */
/* = Image file routines = */
/* ======================= */

/* ---------------------------------------------------------------------- */

/* load_header : read image header into ihdrP from fio for image sdP
** Try to automatically determine the image file format.
** Some image formats have no magic numbers, so this is not
** guarenteed to work.  In general, it will, but in particular it
** would be possible to contrive a file that would fool this routine.
**
** Among the pertinant info set by this routine is a pointer to
** an appropriate routine to load the rest of the file, i.e. the image
** body.  Also sets number of rows and columns, and image type.
*/
static int
load_header(image_hdr_t *ihdrP, screendata_t *sdP, FILE *fio)
{
  unsigned char magic[2];
  short         format;

  if(feof(fio)) return(EOF);

  /* Read the magic number */
  if(get_block(fio, magic, 2)) return (EOF);

  /* Try each of several image file formats.
  ** Since timing matters, put these in the order of the most
  ** likely images to be encountered to improve speed.
  */
  if(magic[0]=='P') {
    /* File is probably one of the PBMplus formats */

    /* Create the format word */
    ihdrP->image_type = magic[0]*256 + magic[1];

    load_pnm_header(ihdrP, sdP, fio);

  } else if((format = magic[0] + 256 * magic[1]) == 0x1234) {
    /* File is probably a PIC or CLP file */

    ihdrP->image_type = format;

    load_pic_header(ihdrP, sdP, fio);

  } else {
    /* Try loading the image as a Targa */
    int tga_return;

    /* Rewind the file to the beginning so we can reread it all */
    fseek(fio, 0, 0);
    if(tga_return=load_tga_header(ihdrP, sdP, fio)) {
      logmesg("load_header: bad magic number or unsupported image\n");
      logmesg(" magic number in byte 1 is [%u] = %c\n", magic[0], magic[0]);
      logmesg(" magic number in byte 2 is [%u] = %c\n", magic[1], magic[1]);
      logmesg(" TGA returned %i\n", tga_return);
      return(EOF);
    }
  }

  return(0);
}

/* ---------------------------------------------------------------------- */

/* load_palette_file : read and install a color map from file pfn
*/
int
load_palette_file(char *pfn)
{
  int           ri;
  image_hdr_t   img_hdr;
  screendata_t  sd;
  FILE *        fio;

  if((fio=open_file(pfn)) == NULL) {
    logmesg("\nload_palette_file: cannot open palette file %s.\n", pfn);
    return(EOF);
  }

  /* Read the PPM header */
  if((ri=load_header(&img_hdr, &sd, fio)) ==EOF) {
    logmesg("read error in palette header, file \"%s\"\n", pfn);
    return(EOF);
  }

  /* Read each palette color value and install it */
  if(   img_hdr.image_type==PPM_FORMAT || img_hdr.image_type==RPPM_FORMAT
     || img_hdr.image_type==PGM_FORMAT || img_hdr.image_type==RPGM_FORMAT) {
    load_ppm_palette(fio, &img_hdr);
    if(img_hdr.hdr != NULL) {
      free(img_hdr.hdr);
    }
  } else {
    logmesg("load_palette_file: not PPM, format=%i\n", img_hdr.image_type);
  }

  fclose(fio);
  return(0);
}

/* ---------------------------------------------------------------------- */

/* load_image : read an image from fio into image buffer sdP
*/
int
load_image(screendata_t *sdP, FILE *fio)
{
  register int row;
  register int bytes_per_row;
  static image_hdr_t  img_hdr;

  img_hdr = gm_hdr_init;

  /* Read the header */
  if(load_header(&img_hdr, sdP, fio)) return (EOF);

  /* Initialize screen data if necessary */
  if(  (img_hdr.num_rows!=sdP->num_rows)
     ||(img_hdr.num_cols!=sdP->num_cols))
  {
    free_movie(sdP);
    if(init_movie(img_hdr.num_rows, img_hdr.num_cols, sdP)) {
      logmesg("\nload_image: failed to init_movie\n");
      bye_bye(2);
    }
  }

  /* Set the header and allocate memory for each portion of the image */
  bytes_per_row = MOVIE_IMAGE_SIZE(0, 0, img_hdr.num_cols-1, 0);
  sdP->header_size = bytes_per_row-img_hdr.num_cols;

  for(row=0; row < sdP->num_rows; row++) {
    /* Allocate memory for the row, but only if necessary */
    if(sdP->data[row] == NULL) {
      sdP->data[row] = (unsigned char *)malloc((size_t)bytes_per_row);
      if( sdP->data[row] == NULL)
      {
        reset_display();
        logmesg("load_image: Unable to allocate %i bytes for row %i/%i.\n",
                bytes_per_row, row, sdP->num_rows);
        bye_bye(3);
      }
    }

    /* Set the image header */
#if defined(MSC)
    if(row==0) {
      _getimage(0, 0, img_hdr.num_cols-1, 0, sdP->data[row]);
    } else {
      memcpy(sdP->data[row],sdP->data[0], (size_t)sdP->header_size);
    }
#elif defined(TC)
    ((UWORD *)sdP->data[row])[0] = (UWORD)(img_hdr.num_cols*COL_FACTOR);
    ((UWORD *)sdP->data[row])[1] = (UWORD)(1);
#endif /* architecture */
  }

  /* Load the image body */ /* MJG 6apr94 */
  (*(img_hdr.load_img)) (&img_hdr, sdP, fio);

  /* Do image transformations */
  /* But make sure we don't go recursive! */
  if(!gm_in_do_xform) do_xform(sdP);

  if(img_hdr.hdr != NULL) {
    free(img_hdr.hdr);
    img_hdr.hdr = NULL;
  }
  return(0);
}

/* ====================================================================== */
/* = Image manipulation routines = */
/* =============================== */

/* image_hflip : horizontally flip image sdP, in place.
*/
void
image_hflip(screendata_t *sdP)
{
  register int col, row;
  register int maxcol;
  char tmp;

  maxcol=sdP->num_cols - 1;

  for(row=0; row < sdP->num_rows; row++) {
    for(col= maxcol/2; col >= 0; col--) {
      tmp = sdP->data[row][sdP->header_size + col];
      sdP->data[row][sdP->header_size + col] =
        sdP->data[row][sdP->header_size + maxcol-col];
      sdP->data[row][sdP->header_size + maxcol-col] = tmp;
    }
  }
}

/* ---------------------------------------------------------------------- */

/* image_vflip : vertically flip image sdP, in place.
*/
void
image_vflip(screendata_t *sdP)
{
  register int  row;
  register int  maxrow;
  unsigned char *tmp;

  maxrow=sdP->num_rows - 1;

  for(row=maxrow/2; row >= 0; row--) {
    tmp = sdP->data[row];
    sdP->data[row] = sdP->data[maxrow-row];
    sdP->data[maxrow-row] = tmp;
  }
}
