#ifndef _SHOWTIME_H_INCLUDED_
#define _SHOWTIME_H_INCLUDED_
/*
** showtime.h -- movie viewer structures prototypes and macros
**
** Michael J. Gourlay
** Rob P. Zimmer II
*/

#include <fcntl.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>

#include "version.h"

#if defined(MSC)
#include <io.h>
#include <conio.h>
#include <process.h>
#include <graph.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <errno.h>
#include <bios.h>
#elif defined(TC)
#include <io.h>
#include <conio.h>
#include <process.h>
#include <graphics.h>
#include <alloc.h>
#include <bios.h>
#endif /* architecture */

/* ---------------------------------------------------------------------- */

/* screendata_t: colormapped image data description */
typedef struct screendata_ {
  int header_size;
  int num_rows;
  int num_cols;
  int xoffset;
  int yoffset;
  unsigned char **data;
} screendata_t;

/* ---------------------------------------------------------------------- */

/* Playback mode bitfield */

typedef struct play_mode_ {
  unsigned int play      : 1; /* 1 = play , 0 = pause */
  unsigned int step      : 1; /* 1 = step , 0 = stay */
  unsigned int direction : 1; /* 1 = forward, 0 =reverse */
  unsigned int commands  : 1; /* 1 = read commands, 0 =no commands */
  unsigned int stop      : 1; /* 1 = stop , 0 = go */
} play_mode_t;

/* play means play straight through a movie file
** pause means to pause playing frames
**
** step means play one frame due to a keyboard/mouse event
** stay means pause mode without frame advance
**
** commands means to read commands if paused
** no commands means to only handle mouse and keyboard events if paused
**
** stop means end the program
** go means do not end the program
*/
#define MODE_PLAY        1
#define MODE_PAUSE       0
#define MODE_STEP        1
#define MODE_STAY        0
#define MODE_COMMANDS    1
#define MODE_NO_COMMANDS 0
#define MODE_FORWARD     1
#define MODE_REVERSE     0
#define MODE_STOP        1
#define MODE_GO          0

/* ---------------------------------------------------------------------- */

/* File I/O macros */
#define FIOD         m_in_fp

/* ---------------------------------------------------------------------- */

/* MAX_IMAGES is the maximum number of images per file */
#define MAX_IMAGES 1000

/* ---------------------------------------------------------------------- */

#if defined(MSC)

#define MOVIE_IMAGE_SIZE(x1, y1, x2, y2) \
        _imagesize((short)(x1), (short)(y1), (short)(x2),(short)(y2))
#define TICK_TOCK(tv) _bios_timeofday(_TIME_GETCLOCK, &(tv));
#define TICK_TIME 18.2
#define TICK_WRAP 1572480
#define COL_FACTOR 8
#define DELAY(t)
#define DIFFTIME(t1, t0) ((t1) - (t0))
#define KBHIT() kbhit()

#elif defined(TC)

#define MOVIE_IMAGE_SIZE(x1, y1, x2, y2) imagesize(x1, y1, x2, y2)
#define TICK_TOCK(tv) ((tv)=biostime(0/*get time*/, (tv)));
#define TICK_TIME 18.2
#define TICK_WRAP 1572480
#define COL_FACTOR 1
#define GMODE_T int
#define GMODE_V gmode
#define DELAY(t)
#define DIFFTIME(t1, t0) ((t1) - (t0))
#define KBHIT() kbhit()

#elif defined(X11)

#define MOVIE_IMAGE_SIZE(x1, y1, x2, y2) (((y2)-(y1)+1)*((x2)-(x1)+1))
#define TICK_TOCK(tv) ((tv)=time(NULL))
#define TICK_TIME 1.0
#define TICK_WRAP 0
#define DELAY(t) sleep(t)
#ifdef SYSV
#define DIFFTIME(t1, t0) difftime((t1), (t0))
#else
#define DIFFTIME(t1, t0) ((t1) - (t0))
#endif /* SYSV */
#define KBHIT() 0

#endif /* architecture */

/* ---------------------------------------------------------------------- */

#define FALSE    0
#define TRUE     (!(FALSE))
#define UWORD    unsigned short int

/* UTILITY MACROS */

#define SIGN(y)  (((y)>0) ? +1 : -1)
#define MIN2(x1,x2)  ((x1)>(x2) ? (x2) : (x1))
#define MAX2(x1,x2)  ((x1)>(x2) ? (x1) : (x2))
#define MAX3(x1,x2,x3)  MAX2(x1, MAX2(x2,x3))
#define LIMIT(x1,x2,x3)  MAX2(x1, MIN2(x3,x2))
#define STREQ(s1, s2) (!strcmp(s1, s2))

/* ====================================================================== */

/* GLOBAL VARIABLES */

/* depth of do_xform calls */
extern int gm_in_do_xform;

/* ====================================================================== */

/* PROTOTYPES */

/* Error logging */
extern void logmesg(char *fmt, ...);

/* Clean up and exit */
extern void bye_bye(int rv);

/* init_movie: allocate memory for the row pointers */
extern int init_movie(int num_rows, int num_cols, screendata_t *sdP);

/* free_movie : free allocated memory for screen data */
extern void free_movie(screendata_t *);

/* do_xform: execute "command macro", applied to each image as it is loaded */
extern void do_xform(screendata_t *sdP);

/* ====================================================================== */

#endif /* _SHOWTIME_H_INCLUDED_ */
